/* ************************************************************************** */
// File       : vertexbuffer.h
// Programmer : Tobin Schwaiger-Hastanan
// Copyright (c) 1997-2002 Tobin Schwaiger-Hastanan
// Notes      : for flipcode.com's COTD.
/* ************************************************************************** */
#ifndef _VERTEXBUFFER_DAWT_H_INCLUDED_
#define _VERTEXBUFFER_DAWT_H_INCLUDED_

/**
 * This is an abstract class that provides an interface on top of various
 * implementations of vertex buffers or similar structures.  I currently
 * use this class for interfacing with D3D Vertex Buffers and OpenGL
 * vertex arrays.
 *
 * @author Tobin Schwaiger-Hastanan
 */
class CVertexBuffer
{
public:
    enum
    {
        FORMAT_XYZ           = 0x0001, /** Untransformed XYZ value set. */
        FORMAT_NORMAL        = 0x0002, /** XYZ normal value set */
        FORMAT_DIFFUSE       = 0x0004, /** RGBA diffuse color value set */
        FORMAT_SPECULAR      = 0x0008, /** RGBA specular color value set */
        FORMAT_XYZRHW        = 0x0010, /** transformed XYZRHW value set. */
        FORMAT_TEXVERTEXFLAG = 0x8000, /** Texture vertex flag. */           
    };

    /**
      * This method should be provided with the vertex format when creating the 
      * vertex buffer.  This will include FORMAT_TEXVERTEXFORMAT and encode
      * the number of texture vertex needed by the developer.
      * example: FORMAT_XYZ | FORMAT_NORMAL | FORMAT_DIFFUSE | FORMAT_TEXVERTEX( 1 )
      */
    int FORMAT_TEXVERTEX( int nTexVertex )
    {
        return FORMAT_TEXVERTEXFLAG | (nTexVertex << 8 );
    }

    /**
      * supported primitives
      */
    enum PRIMITIVE
    {
        PRIMITIVE_POINTLIST,     /** Specifies a point list  */
        PRIMITIVE_LINELIST,      /** Specifies a line list   */
        PRIMITIVE_LINESTRIP,     /** Specifies a line strip  */
        PRIMITIVE_TRIANGLELIST,  /** Specifies a triangle list  */
        PRIMITIVE_TRIANGLESTRIP, /** Specifies a triangle strip */
        PRIMITIVE_TRIANGLEFAN,   /** Specifies a triangle fan   */
    };

public:
    virtual ~CVertexBuffer() { } // I'm sure I will get some flak for this.
    

    /**
      * Allows the developer to specify the primitive type for rendering.      
      * @param primitive Primitive type assigned to vertex buffer.
      */
    virtual void setPrimitive( PRIMITIVE& primitive ) = 0;

    /**
      * Returns the primitive type associated to the vertex buffer.      
      * @return Primitive type associated to the vertex buffer.
      */
    virtual PRIMITIVE getPrimitive() = 0;

    /**
      * Locks the vertex buffer (usually for writing)...      
      * @return reference to current vertex buffer object.
      */
    virtual CVertexBuffer& lock()   = 0;

    /**
      * Unlocks the vertex buffer.      
      * @return reference to current vertex buffer object.
      */
    virtual CVertexBuffer& unlock() = 0;

    /**
      * Determines if the vertex buffer is locked or not.      
      * @return boolean value specifying if vertex buffer is locked or not.
      */
    virtual bool isLocked() = 0;

    /**
      * Tells the vertex buffer it will be sent vertex data. (ala OpenGL)
      *
      * @param primitive primitive that describes incoming data. 
      *                  [Default: PRIMITIVE_PPINTLIST]      
      * @return boolean value specifying if vertex buffer is locked or not.
      */
    virtual CVertexBuffer& begin( PRIMITIVE primitive = PRIMITIVE_POINTLIST ) = 0;

    /**
      * Tells the vertex buffer no more vertex data will be sent. (ala OpenGL)
      *
      * @param primitive primitive that describes incoming data. 
      *                  [Default: PRIMITIVE_PPINTLIST]      
      * @return boolean value specifying if vertex buffer is locked or not.
      */
    virtual CVertexBuffer& end()   = 0;

    /**
      * Returns a pointer to the raw vertex data.      
      * This should only be called when vertex buffer is locked.      
      * @return pointer to raw vertex data.
      */
    virtual void* getVertexData() = 0;        
    
    /**
      * returns the number of vertex currently in the vertex buffer
      * @return number of vertex in vertex buffer.
      */
    virtual int getVertexCount()     = 0;

    /**
      * returns the maximum number of vertex the vertex buffer can handle.
      * @return maximum number of vertex the vertex buffer can handle.
      */
    virtual int getMaxVertexCount()  = 0;

    /**
      * returns the vertex format of the vertex buffer. (see FORMAT enumeration)
      * @return vertex format of the vertex buffer. 
      */
    virtual unsigned long getVertexFormat() = 0;

    /**
      * returns the width in bytes of a single vertex described by the format.
      * @return width in bytes of a single vertex described by the format.
      */
    virtual unsigned long getVertexStride() = 0;

    /**
      * Specifies untransformed vertex data. Should be last call
      * for each vertex. That is after the normal, color, and
      * texture coordinate calls are made for the vertex.
      * Note: should be used when format includes FORMAT_XYZ
      *
      * @param x x coordinate.
      * @param y y coordinate.
      * @param z z coordinate.
      */
    virtual CVertexBuffer& vertex( float x, float y, float z ) = 0;

    /**
      * Specifies transformed vertex data. Should be last call
      * for each vertex. That is after the normal, color, and
      * texture coordinate calls are made for the vertex.
      * Note: should be used when format includes FORMAT_XYZRHW
      *
      * @param x x coordinate.
      * @param y y coordinate.
      * @param z z coordinate.
      * @param w w coordinate.
      */
    virtual CVertexBuffer& vertex( float x, float y, float z, float w ) = 0;

    /**
      * Specifies vertex normal data.
      * Note: should be used when format includes FORMAT_NORMAL
      *
      * @param x Normal x value.
      * @param y Normal y value.
      * @param z Normal z value.
      */
    virtual CVertexBuffer& normal( float x, float y, float z ) = 0;

    /**
      * Specifies diffuse color data.
      * Note: should be used when format includes FORMAT_DIFFUSE
      *
      * @param r Red color value.
      * @param g Green color value.
      * @param b Blue color value.
      * @param a Alpha color value.
      */
    virtual CVertexBuffer& diffuse( float r, float g, float b, float a = 1.0f )  = 0;

    /**
      * Specifies specular color data.
      * Note: should be used when format includes FORMAT_SPECULAR
      *
      * @param r Red color value.
      * @param g Green color value.
      * @param b Blue color value.
      * @param a Alpha color value.
      */
    virtual CVertexBuffer& specular( float r, float g, float b, float a = 1.0f ) = 0;

    /**
      * Specifies texture vertex. Successive calls should be made to this
      * method if more than one set of texture coordinates need to be 
      * specified.
      * Note: should be used when format includes FORMAT_TEXTUREFLAG
      *
      * @param u U texture coordinate.
      * @param v V texture coordinate.
      */
    virtual CVertexBuffer& texvertex( float u, float v ) = 0;
};

#endif